@@ -27,14 +27,13 @@
 #include <QBitmap>
 #include <QSGTexture>
 #include <QDebug>
-#include <QSGSimpleTextureNode>
-#include <QSGTexture>
 #include <QSharedPointer>
 #include <QtQml>
 #include <QQuickImageProvider>
 #include <QGuiApplication>
 #include <QPointer>
 #include <QPainter>
+#include <QScreen>
 
 class ManagedTextureNode : public QSGSimpleTextureNode
 {
@@ -124,7 +123,7 @@ QSharedPointer<QSGTexture> ImageTexturesCache::loadTexture(QQuickWindow *window,
 
 QSharedPointer<QSGTexture> ImageTexturesCache::loadTexture(QQuickWindow *window, const QImage &image)
 {
-    return loadTexture(window, image, 0);
+    return loadTexture(window, image, nullptr);
 }
 
 Q_GLOBAL_STATIC(ImageTexturesCache, s_iconImageCache)
@@ -143,6 +142,10 @@ DesktopIcon::DesktopIcon(QQuickItem *parent)
         m_changed = true;
         update();
     });
+    connect(this, &QQuickItem::enabledChanged, this, [this]() {
+        m_changed = true;
+        update();
+    });
 }
 
 
@@ -157,6 +160,7 @@ void DesktopIcon::setSource(const QVariant &icon)
     }
     m_source = icon;
     m_changed = true;
+    m_monochromeHeuristics.clear();
 
     if (!m_theme) {
         m_theme = static_cast<Kirigami::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::PlatformTheme>(this, true));
@@ -168,6 +172,19 @@ void DesktopIcon::setSource(const QVariant &icon)
         });
     }
 
+    if (icon.type() == QVariant::String) {
+        const QString iconSource = icon.toString();
+        m_isMaskHeuristic = (iconSource.endsWith(QStringLiteral("-symbolic"))
+                            || iconSource.endsWith(QStringLiteral("-symbolic-rtl"))
+                            || iconSource.endsWith(QStringLiteral("-symbolic-ltr")));
+        emit isMaskChanged();
+    }
+
+    if (m_networkReply) {
+        //if there was a network query going on, interrupt it
+        m_networkReply->close();
+    }
+    m_loadedImage = QImage();
     update();
     emit sourceChanged();
 }
@@ -177,18 +194,6 @@ QVariant DesktopIcon::source() const
     return m_source;
 }
 
-void DesktopIcon::setEnabled(const bool enabled)
-{
-    if (enabled == QQuickItem::isEnabled()) {
-        return;
-    }
-    QQuickItem::setEnabled(enabled);
-    m_changed = true;
-    update();
-    emit enabledChanged();
-}
-
-
 void DesktopIcon::setActive(const bool active)
 {
     if (active == m_active) {
@@ -233,6 +238,7 @@ void DesktopIcon::setIsMask(bool mask)
     }
 
     m_isMask = mask;
+    m_isMaskHeuristic = mask;
     m_changed = true;
     update();
     emit isMaskChanged();
@@ -240,7 +246,7 @@ void DesktopIcon::setIsMask(bool mask)
 
 bool DesktopIcon::isMask() const
 {
-    return m_isMask;
+    return m_isMask || m_isMaskHeuristic;
 }
 
 void DesktopIcon::setColor(const QColor &color)
@@ -294,7 +300,7 @@ QSGNode* DesktopIcon::updatePaintNode(QSGNode* node, QQuickItem::UpdatePaintNode
         return Q_NULLPTR;
     }
 
-    if (m_changed || node == 0) {
+    if (m_changed || node == nullptr) {
         QImage img;
         const QSize itemSize(width(), height());
         QRect nodeRect(QPoint(0,0), itemSize);
@@ -334,13 +340,23 @@ QSGNode* DesktopIcon::updatePaintNode(QSGNode* node, QQuickItem::UpdatePaintNode
                 img = QImage(size, QImage::Format_Alpha8);
                 img.fill(Qt::transparent);
             }
-            if (img.size() != size){
+            if (img.size() != size) {
                 // At this point, the image will already be scaled, but we need to output it in
                 // the correct aspect ratio, painted centered in the viewport. So:
                 QRect destination(QPoint(0, 0), img.size().scaled(itemSize, Qt::KeepAspectRatio));
                 destination.moveCenter(nodeRect.center());
                 nodeRect = destination;
             }
+            
+            const QColor tintColor = !m_color.isValid() || m_color == Qt::transparent ? (m_selected ? m_theme->highlightedTextColor() : m_theme->textColor()) : m_color;
+
+            //TODO: initialize m_isMask with icon.isMask()
+            if (tintColor.alpha() > 0 && (isMask() || guessMonochrome(img))) {
+                QPainter p(&img);
+                p.setCompositionMode(QPainter::CompositionMode_SourceIn);
+                p.fillRect(img.rect(), tintColor);
+                p.end();
+            }
         }
         m_changed = false;
 
@@ -370,7 +386,7 @@ void DesktopIcon::geometryChanged(const QRectF &newGeometry, const QRectF &oldGe
 }
 
 void DesktopIcon::handleFinished(QNetworkAccessManager* qnam, QNetworkReply* reply) {
-    if (reply->error() == QNetworkReply::NoError) {
+    if (reply && reply->error() == QNetworkReply::NoError) {
         const QUrl possibleRedirectUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
         if (!possibleRedirectUrl.isEmpty()) {
             const QUrl redirectUrl = reply->url().resolved(possibleRedirectUrl);
@@ -382,9 +398,9 @@ void DesktopIcon::handleFinished(QNetworkAccessManager* qnam, QNetworkReply* rep
             reply->deleteLater();
             QNetworkRequest request(possibleRedirectUrl);
             request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::PreferCache);
-            QNetworkReply* newReply = qnam->get(request);
-            connect(newReply, &QNetworkReply::readyRead, this, [this, newReply](){ handleReadyRead(newReply); });
-            connect(newReply, &QNetworkReply::finished, this, [this, qnam, newReply](){ handleFinished(qnam, newReply); });
+            m_networkReply = qnam->get(request);
+            connect(m_networkReply.data(), &QNetworkReply::readyRead, this, [this](){handleReadyRead(m_networkReply); });
+            connect(m_networkReply.data(), &QNetworkReply::finished, this, [this, qnam](){handleFinished(qnam, m_networkReply); });
             return;
         }
     }
@@ -392,25 +408,26 @@ void DesktopIcon::handleFinished(QNetworkAccessManager* qnam, QNetworkReply* rep
 
 void DesktopIcon::handleReadyRead(QNetworkReply* reply)
 {
-    if (reply->attribute(QNetworkRequest::RedirectionTargetAttribute).isNull()) {
+    if (reply && reply->attribute(QNetworkRequest::RedirectionTargetAttribute).isNull()) {
         // We're handing the event loop back while doing network work, and it turns out
         // this fairly regularly results in things being deleted under us. So, just
         // handle that and crash less :)
         QPointer<DesktopIcon> me(this);
+        QPointer<QNetworkReply> guardedReply(reply);
         QByteArray data;
         do {
-            data.append(reply->read(32768));
+            data.append(guardedReply->read(32768));
             // Because we are in the main thread, this could be potentially very expensive, so let's not block
             qApp->processEvents();
-            if(!me) {
+            if(!me || !guardedReply) {
                 return;
             }
-        } while(!reply->atEnd());
+        } while(!guardedReply->atEnd());
         m_loadedImage = QImage::fromData(data);
         if (m_loadedImage.isNull()) {
             // broken image from data, inform the user of this with some useful broken-image thing...
             const QSize size = QSize(width(), height()) * (window() ? window()->devicePixelRatio() : qApp->devicePixelRatio());
-            m_loadedImage = QIcon::fromTheme("unknown").pixmap(size, iconMode(), QIcon::On).toImage();
+            m_loadedImage = QIcon::fromTheme(m_fallback).pixmap(size, iconMode(), QIcon::On).toImage();
         }
         m_changed = true;
         update();
@@ -421,10 +438,17 @@ QImage DesktopIcon::findIcon(const QSize &size)
 {
     QImage img;
     QString iconSource = m_source.toString();
-    if (iconSource.startsWith("image://")){
+
+    if (iconSource.startsWith(QLatin1String("image://"))) {
         QUrl iconUrl(iconSource);
         QString iconProviderId = iconUrl.host();
         QString iconId = iconUrl.path();
+
+        // QRC paths are not correctly handled by .path()
+        if (iconId.size() >=2 && iconId.startsWith(QLatin1String("/:"))) {
+            iconId = iconId.remove(0, 1);
+        }
+
         QSize actualSize;
         QQuickImageProvider* imageProvider = dynamic_cast<QQuickImageProvider*>(
                     qmlEngine(this)->imageProvider(iconProviderId));
@@ -443,42 +467,49 @@ QImage DesktopIcon::findIcon(const QSize &size)
             //will have to investigate this more
             break;
         }
-    } else if(iconSource.startsWith("http://") || iconSource.startsWith("https://")) {
+    } else if(iconSource.startsWith(QLatin1String("http://")) || iconSource.startsWith(QLatin1String("https://"))) {
         if(!m_loadedImage.isNull()) {
             return m_loadedImage.scaled(size, Qt::KeepAspectRatio, m_smooth ? Qt::SmoothTransformation : Qt::FastTransformation );
         }
+        const auto url = m_source.toUrl();
         QQmlEngine* engine = qmlEngine(this);
         QNetworkAccessManager* qnam;
-        if (engine && (qnam = qmlEngine(this)->networkAccessManager())) {
-            QNetworkRequest request(m_source.toUrl());
+        if (engine && (qnam = engine->networkAccessManager()) && (!m_networkReply || m_networkReply->url() != url)) {
+            QNetworkRequest request(url);
             request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::PreferCache);
-            QNetworkReply* reply = qnam->get(request);
-            connect(reply, &QNetworkReply::readyRead, this, [this, reply](){ handleReadyRead(reply); });
-            connect(reply, &QNetworkReply::finished, this, [this, qnam, reply](){ handleFinished(qnam, reply); });
+            m_networkReply = qnam->get(request);
+            connect(m_networkReply.data(), &QNetworkReply::readyRead, this, [this](){ handleReadyRead(m_networkReply); });
+            connect(m_networkReply.data(), &QNetworkReply::finished, this, [this, qnam](){ handleFinished(qnam, m_networkReply); });
         }
         // Temporary icon while we wait for the real image to load...
-        img = QIcon::fromTheme("image-x-icon").pixmap(size, iconMode(), QIcon::On).toImage();
+        img = QIcon::fromTheme(QStringLiteral("image-x-icon")).pixmap(size, iconMode(), QIcon::On).toImage();
     } else {
-        if (iconSource.startsWith("qrc:/")){
+        if (iconSource.startsWith(QLatin1String("qrc:/"))) {
             iconSource = iconSource.mid(3);
+        } else if (iconSource.startsWith(QLatin1String("file:/"))) {
+            iconSource = QUrl(iconSource).path();
         }
-        QIcon icon;
-        const QColor tintColor = m_color == Qt::transparent ? m_theme->textColor() : m_color;
-        const bool isPath = iconSource.contains("/");
 
+        QIcon icon;
+        const bool isPath = iconSource.contains(QLatin1String("/"));
         if (isPath) {
             icon = QIcon(iconSource);
         } else {
-            icon = m_theme->iconFromTheme(iconSource, tintColor);
+            if (icon.isNull()) {
+                icon = m_theme->iconFromTheme(iconSource, m_color);
+            }
         }
-        if (!icon.availableSizes().isEmpty()){
+        if (!icon.isNull()) {
             img = icon.pixmap(size, iconMode(), QIcon::On).toImage();
-            if (m_isMask || icon.isMask() || (isPath && tintColor != Qt::transparent)) {
+
+            /*const QColor tintColor = !m_color.isValid() || m_color == Qt::transparent ? (m_selected ? m_theme->highlightedTextColor() : m_theme->textColor()) : m_color;
+
+            if (m_isMask || icon.isMask() || iconSource.endsWith(QStringLiteral("-symbolic")) || iconSource.endsWith(QStringLiteral("-symbolic-rtl")) || iconSource.endsWith(QStringLiteral("-symbolic-ltr")) || guessMonochrome(img)) {
                 QPainter p(&img);
                 p.setCompositionMode(QPainter::CompositionMode_SourceIn);
                 p.fillRect(img.rect(), tintColor);
                 p.end();
-            }
+            }*/
         }
     }
     return img;
@@ -495,3 +526,77 @@ QIcon::Mode DesktopIcon::iconMode() const
     }
     return QIcon::Normal;
 }
+
+bool DesktopIcon::guessMonochrome(const QImage &img)
+{
+    //don't try for too big images
+    if (img.width() >= 256 || m_theme->supportsIconColoring()) {
+        return false;
+    }
+    // round size to a standard size. hardcode as we can't use KIconLoader
+    int stdSize;
+    if (img.width() <= 16) {
+        stdSize = 16;
+    } else if (img.width() <= 22) {
+        stdSize = 22;
+    } else if (img.width() <= 24) {
+        stdSize = 24;
+    } else if (img.width() <= 32) {
+        stdSize = 32;
+    } else if (img.width() <= 48) {
+        stdSize = 48;
+    } else if (img.width() <= 64) {
+        stdSize = 64;
+    } else {
+        stdSize = 128;
+    }
+
+    auto findIt = m_monochromeHeuristics.constFind(stdSize);
+    if (findIt != m_monochromeHeuristics.constEnd()) {
+        return findIt.value();
+    }
+
+    QHash<int, int> dist;
+    int transparentPixels = 0;
+    int saturatedPixels = 0;
+    for(int x=0; x < img.width(); x++) {
+        for(int y=0; y < img.height(); y++) {
+            QColor color = QColor::fromRgba(qUnpremultiply(img.pixel(x, y)));
+            if (color.alpha() < 100) {
+                ++transparentPixels;
+                continue;
+            } else if (color.saturation() > 84) {
+                ++saturatedPixels;
+            }
+            dist[qGray(color.rgb())]++;
+        }
+    }
+
+    QMultiMap<int, int> reverseDist;
+    auto it = dist.constBegin();
+    std::vector<qreal> probabilities(dist.size());
+    qreal entropy = 0;
+    while (it != dist.constEnd()) {
+        reverseDist.insertMulti(it.value(), it.key());
+        qreal probability = qreal(it.value()) / qreal(img.size().width() * img.size().height() - transparentPixels);
+        entropy -= probability * log(probability) / log(255);
+        ++it;
+    }
+
+    // Arbitrarly low values of entropy and colored pixels
+    m_monochromeHeuristics[stdSize] = saturatedPixels <= (img.size().width()*img.size().height() - transparentPixels) * 0.3 && entropy <= 0.3;
+    return m_monochromeHeuristics[stdSize];
+}
+
+QString DesktopIcon::fallback() const
+{
+    return m_fallback;
+}
+
+void DesktopIcon::setFallback(const QString& fallback)
+{
+    if (m_fallback != fallback) {
+        m_fallback = fallback;
+        Q_EMIT fallbackChanged(fallback);
+    }
+}
