@@ -62,6 +62,7 @@ public:
     QColor positiveTextColor;
 
     QColor backgroundColor;
+    QColor alternateBackgroundColor;
     QColor highlightColor;
 
     QColor focusColor;
@@ -81,6 +82,7 @@ public:
     QColor customPositiveTextColor;
 
     QColor customBackgroundColor;
+    QColor customAlternateBackgroundColor;
     QColor customHighlightColor;
 
     QColor customFocusColor;
@@ -91,6 +93,7 @@ public:
     QFont font;
     bool m_inherit = true;
     bool m_init = true;
+    bool m_supportsIconColoring = false;
 
     static KirigamiPluginFactory *s_pluginFactory;
 };
@@ -159,6 +162,7 @@ void PlatformThemePrivate::findParentStyle()
                 q->setCustomNeutralTextColor(t->d->customNeutralTextColor);
                 q->setCustomPositiveTextColor(t->d->customPositiveTextColor);
                 q->setCustomBackgroundColor(t->d->customBackgroundColor);
+                q->setCustomAlternateBackgroundColor(t->d->customAlternateBackgroundColor);
                 q->setCustomHighlightColor(t->d->customHighlightColor);
                 q->setCustomFocusColor(t->d->customFocusColor);
                 q->setCustomHoverColor(t->d->customHoverColor);
@@ -223,21 +227,6 @@ void PlatformTheme::setColorSet(PlatformTheme::ColorSet colorSet)
     for (PlatformTheme *t : qAsConst(d->m_childThemes)) {
         if (t->inherit()) {
             t->setColorSet(colorSet);
-           /* if (colorSet == Custom) {
-                t->setCustomTextColor(textColor());
-                t->setCustomDisabledTextColor(disabledTextColor());
-                t->setCustomHighlightedTextColor(highlightedTextColor());
-                t->setCustomActiveTextColor(activeTextColor());
-                t->setCustomLinkColor(linkColor());
-                t->setCustomVisitedLinkColor(visitedLinkColor());
-                t->setCustomNegativeTextColor(negativeTextColor());
-                t->setCustomNeutralTextColor(neutralTextColor());
-                t->setCustomPositiveTextColor(positiveTextColor());
-                t->setCustomBackgroundColor(backgroundColor());
-                t->setCustomHighlightColor(highlightColor());
-                t->setCustomFocusColor(focusColor());
-                t->setCustomHoverColor(hoverColor());
-            }*/
         }
     }
 
@@ -320,6 +309,11 @@ QColor PlatformTheme::backgroundColor() const
     return d->customBackgroundColor.isValid() ? d->customBackgroundColor : d->backgroundColor;
 }
 
+QColor PlatformTheme::alternateBackgroundColor() const
+{
+    return d->customAlternateBackgroundColor.isValid() ? d->customAlternateBackgroundColor : d->alternateBackgroundColor;
+}
+
 QColor PlatformTheme::activeTextColor() const
 {
     return d->customActiveTextColor.isValid() ? d->customActiveTextColor : d->activeTextColor;
@@ -391,6 +385,16 @@ void PlatformTheme::setBackgroundColor(const QColor &color)
     d->setColorCompressTimer->start();
 }
 
+void PlatformTheme::setAlternateBackgroundColor(const QColor &color)
+{
+    if (d->alternateBackgroundColor == color) {
+        return;
+    }
+
+    d->alternateBackgroundColor = color;
+    d->setColorCompressTimer->start();
+}
+
 void PlatformTheme::setHighlightColor(const QColor &color)
 {
     if (d->highlightColor == color) {
@@ -549,6 +553,17 @@ void PlatformTheme::setCustomBackgroundColor(const QColor &color)
     d->setColorCompressTimer->start();
 }
 
+void PlatformTheme::setCustomAlternateBackgroundColor(const QColor &color)
+{
+    if (d->customAlternateBackgroundColor == color) {
+        return;
+    }
+
+    d->customAlternateBackgroundColor = color;
+    PROPAGATECUSTOMCOLOR(CustomAlternateBackgroundColor, color)
+    d->setColorCompressTimer->start();
+}
+
 void PlatformTheme::setCustomHighlightColor(const QColor &color)
 {
     if (d->customHighlightColor == color) {
@@ -680,12 +695,18 @@ void PlatformTheme::setPalette(const QPalette &palette)
 QIcon PlatformTheme::iconFromTheme(const QString &name, const QColor &customColor)
 {
     QIcon icon = QIcon::fromTheme(name);
-    if (!icon.isNull() && (name.endsWith(QLatin1String("-symbolic")) || customColor != Qt::transparent)) {
-        icon.setIsMask(true);
-    }
     return icon;
 }
 
+bool PlatformTheme::supportsIconColoring() const
+{
+    return d->m_supportsIconColoring;
+}
+
+void PlatformTheme::setSupportsIconColoring(bool support)
+{
+    d->m_supportsIconColoring = support;
+}
 
 
 PlatformTheme *PlatformTheme::qmlAttachedProperties(QObject *object)
